import type { QueryClient, UseMutationOptions } from '@tanstack/react-query';
import type { TRPCClientErrorLike, TRPCUntypedClient } from '@trpc/client';
import type { DistributiveOmit, MaybePromise } from '@trpc/server/unstable-core-do-not-import';
import type { ResolverDef, TRPCMutationKey, TRPCQueryBaseOptions, TRPCQueryOptionsResult } from './types';
type ReservedOptions = 'mutationKey' | 'mutationFn';
interface TRPCMutationOptionsIn<TInput, TError, TOutput, TContext> extends DistributiveOmit<UseMutationOptions<TOutput, TError, TInput, TContext>, ReservedOptions>, TRPCQueryBaseOptions {
}
interface TRPCMutationOptionsOut<TInput, TError, TOutput, TContext> extends UseMutationOptions<TOutput, TError, TInput, TContext>, TRPCQueryOptionsResult {
    mutationKey: TRPCMutationKey;
}
export interface TRPCMutationOptions<TDef extends ResolverDef> {
    <TContext = unknown>(opts?: TRPCMutationOptionsIn<TDef['input'], TRPCClientErrorLike<TDef>, TDef['output'], TContext>): TRPCMutationOptionsOut<TDef['input'], TRPCClientErrorLike<TDef>, TDef['output'], TContext>;
}
/**
 * @internal
 */
export interface MutationOptionsOverride {
    onSuccess: (opts: {
        /**
         * Calls the original function that was defined in the query's `onSuccess` option
         */
        originalFn: () => MaybePromise<void>;
        queryClient: QueryClient;
        /**
         * Meta data passed in from the `useMutation()` hook
         */
        meta: Record<string, unknown>;
    }) => MaybePromise<void>;
}
type AnyTRPCMutationOptionsIn = TRPCMutationOptionsIn<unknown, unknown, unknown, unknown>;
type AnyTRPCMutationOptionsOut = TRPCMutationOptionsOut<unknown, unknown, unknown, unknown>;
/**
 * @internal
 */
export declare function trpcMutationOptions(args: {
    mutate: typeof TRPCUntypedClient.prototype.mutation;
    queryClient: QueryClient | (() => QueryClient);
    path: readonly string[];
    opts: AnyTRPCMutationOptionsIn;
    overrides: MutationOptionsOverride | undefined;
}): AnyTRPCMutationOptionsOut;
export {};
//# sourceMappingURL=mutationOptions.d.ts.map