'use strict';

var reactQuery = require('@tanstack/react-query');
var unstableCoreDoNotImport = require('@trpc/server/unstable-core-do-not-import');
var utils = require('./utils.js');

/**
 * @internal
 */ function trpcQueryOptions(args) {
    const { input, query, path, queryKey, opts } = args;
    const queryClient = utils.unwrapLazyArg(args.queryClient);
    const inputIsSkipToken = input === reactQuery.skipToken;
    const queryFn = async (queryFnContext)=>{
        const actualOpts = {
            ...opts,
            trpc: {
                ...opts?.trpc,
                ...opts?.trpc?.abortOnUnmount ? {
                    signal: queryFnContext.signal
                } : {
                    signal: null
                }
            }
        };
        const queryKey = queryFnContext.queryKey;
        const result = await query(...utils.getClientArgs(queryKey, actualOpts));
        if (unstableCoreDoNotImport.isAsyncIterable(result)) {
            return utils.buildQueryFromAsyncIterable(result, queryClient, queryKey);
        }
        return result;
    };
    return Object.assign(reactQuery.queryOptions({
        ...opts,
        queryKey,
        queryFn: inputIsSkipToken ? reactQuery.skipToken : queryFn
    }), {
        trpc: utils.createTRPCOptionsResult({
            path
        })
    });
}

exports.trpcQueryOptions = trpcQueryOptions;
